library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

entity rotator is
    port (
        clk_in : in std_logic;
        reset : in std_logic;
        data_in : in std_logic_vector(511 downto 0);
        shift : in std_logic_vector(8 downto 0);
        data_out : out std_logic_vector(511 downto 0)    
    );
end rotator;

architecture Behavioral of rotator is
    signal r_data_in : std_logic_vector(511 downto 0);
    signal stage1_out : std_logic_vector(511 downto 0);
    signal stage2_out : std_logic_vector(511 downto 0);
    signal stage3_out : std_logic_vector(511 downto 0);
    
    signal stage1_shift : std_logic_vector(8 downto 0);
    signal stage2_shift : std_logic_vector(5 downto 0);
    signal stage3_shift : std_logic_vector(2 downto 0);
    
    component clk_wiz_0
    port
    (
        clk_out          : out    std_logic;
        reset            : in     std_logic;
        clk_in           : in     std_logic
    );
    end component;
    
    signal clk : std_logic;        
begin
    CLK_GEN : clk_wiz_0
       port map ( 
       clk_out => clk,               
       reset => reset,
       clk_in => clk_in
     );

    REG_PROC: process (clk) is
    begin
        if rising_edge(clk) then
            r_data_in <= data_in;
            data_out <= stage3_out;
            stage1_shift <= shift; -- i shift treba da ide sa podacima
            stage2_shift <= stage1_shift(5 downto 0);        
            stage3_shift <= stage2_shift(2 downto 0); 
        end if;
    end process REG_PROC;
    
    STAGE1: process (clk) is
        variable v_s1shift : integer range 0 to 7;
    begin
        if rising_edge(clk) then
            v_s1shift := to_integer(unsigned(stage1_shift(8 downto 6)));
            stage1_out <= r_data_in;
            L_STAGE1: for iShift in 1 to 7 loop
                if v_s1shift = iShift then
                    stage1_out <= r_data_in(iShift*64 - 1 downto 0) & r_data_in(511 downto iShift*64);    
                end if;        
            end loop L_STAGE1;
        end if;
    end process STAGE1;
    
    STAGE2: process (clk) is
        variable v_s2shift : integer range 0 to 7;
    begin
        if rising_edge(clk) then
            v_s2shift := to_integer(unsigned(stage2_shift(5 downto 3)));
            stage2_out <= stage1_out;
            L_STAGE2: for iShift in 1 to 7 loop
                if v_s2shift = iShift then
                    stage2_out <= stage1_out(iShift*8 - 1 downto 0) & stage1_out(511 downto iShift*8);    
                end if;        
            end loop L_STAGE2; 
        end if;           
    end process STAGE2;
    
    STAGE3: process (stage2_out, stage3_shift) is
        variable v_s3shift : integer range 0 to 7;
    begin
        v_s3shift := to_integer(unsigned(stage3_shift(2 downto 0)));
        stage3_out <= stage2_out;
        L_STAGE3: for iShift in 1 to 7 loop
            if v_s3shift = iShift then
                stage3_out <= stage2_out(iShift - 1 downto 0) & stage2_out(511 downto iShift);    
            end if;        
        end loop L_STAGE3;            
    end process STAGE3;
    
end Behavioral;
